require_relative 'methods/required_files'
#====================================
# INTERACTION STEPS
#====================================
# Step used to execute a SQL script on a specific schema
And /^I run sql script "([^\"]*)" on "([^\"]*)" schema$/ do |script, schema|
  begin
    sql = "#{script}"
    DatabaseMethods.execute_select(schema, sql)
    DatabaseMethods.close_connection(schema)
  ensure
  end
end
# Step used to execute SQL script from SQL file on specific schema
Given /^I run sql script from file "([^\"]*)" on "([^\"]*)" schema$/ do |sql_script, schema|
  if sql_script != nil and sql_script != ""
    begin
      File.readlines($sql_dir + sql_script).each do |line|
        if line.nil? || line =~ /^\s*\n*--/
          # puts "\nSQL: " + line;
          next;
        end
        line = line.strip();
        line = line[0..-2];
        #puts "\nSQL: " + line;
        DatabaseMethods.execute_select(schema, line)
        sleep(1);
      end
      DatabaseMethods.close_connection(schema)
    end
  end
end
#====================================
# VERIFICATION STEPS
#====================================
# Step used to check if data exists in database or not
Then /^I should ( not)? see data in table "([^\"]*)" on "([^\"]*)" schema in database as following record:$/ do |negative, table, schema, table_data|
  records = Util.read_table_data_from_steps_with_header(table_data.raw)
  records = JSON.parse(JSON.generate(records))
  puts records
  records.each { |values|
    table_type_obj = Util.generate_condition_statement(values)
    # puts table_type_obj
    begin
      sql= "select count(*) from #{schema}.#{table} where #{table_type_obj}"
      rs = DatabaseMethods.execute_select(schema, sql)
      # puts sql
      obj_array = Array.new
      rs.each { |row|
        row.each_pair { |col, value|
          assert(false) if negative.nil? && value == 0 || !negative.nil? && value == 1
        }
      }
    ensure
      DatabaseMethods.close_connection(schema)
    end
  }
end
# Step used to check if data exists in database or not with condition in file
Then /^I should ( not)? see data in schema "(.*)" database with SQL query in file "(.*)"$/ do |negative, schema, file|
  sql = File.read($test_data_dir + file)
  filepath = ($test_data_dir + file)
  if File.exist?(filepath)
    begin
      rs = DatabaseMethods.execute_select(schema, sql)
      obj_array = Array.new
      rs.each { |row|
        row.each_pair { |col, value|
          assert(false) if negative.nil? && value == 0 || !negative.nil? && value == 1
        }
      }
    ensure
      DatabaseMethods.close_connection(schema)
    end
  else
    raise "*** ERROR: Please check #{filepath} exists."
  end
end

# Step used to get data from database
Then /^I get "(.*)" data from table "([^\"]*)" on "([^\"]*)" schema in database as following record:$/ do |value, table, schema, table_data|
  records = Util.read_table_data_from_steps_with_header(table_data.raw)
  records = JSON.parse(JSON.generate(records))
  records.each { |values|
    table_type_obj = Util.generate_condition_statement(values)
    begin
      sql= "select #{value} from #{table} where #{table_type_obj}"
      sql_count= "select count(*) from #{table} where #{table_type_obj}"
      rs = DatabaseMethods.execute_select(schema, sql)
      record_result= DatabaseMethods.execute_select(schema, sql_count)
      record_result.each { |rows|
        rows.each_pair { |cols, values|
          @@value_count= values
        }
      }
      if @@value_count == 0
        puts ("***WARNING: NOT FOUND #{table_type_obj} in #{schema}")

      else
        rs.each { |row|
          row.each_pair { |cols, values|
            $result=values
          }
        }
      end
    ensure
      DatabaseMethods.close_connection(schema)
    end
  }
end
