require_relative 'methods/required_files'
# Navigate to a particular page
Given(/^I am on the "([^"]*)" page$/) do |page|
  execute_openbrowser(page)
end

#Navigating to the login page
Given /^I am on Login Page$/ do
  execute_openbrowser($_CFWEB['Page Address'])
end

# Wait for the specific time
When /^I wait for (\d+) seconds$/ do |second|
  sleep(second.to_i)
end

# Finds a button or link by id, text or value and clicks it
And /^I click on "([^\"]*)"$/ do |object|
  execute_click(object)
end

# Finds and clicks it with JS
And /^I move mouse to element "([^\"]*)" then click$/ do |object|
  execute_mousehoverandclick(object)
end

# Fill in a text box or text area with a value
And /^I set text on "(.*?)" with "(.*?)"$/ do |object, text|
  text = var_collect(text)
  text = replace_pipe(text)
  execute_settext(object, text)
end

# set state for check box, combo box
And /^I set state on "(.*?)" with "(.*?)"$/ do |object, state|
  execute_setstate(object, state)
end

# Input data to text box, this function support in case user want to input data into many text boxes on the same time
When /^I input data to web form with following values:$/ do |value|
  records = Util.read_table_data_from_steps_with_header(value.raw)
  records = JSON.parse(JSON.generate(records))
  records.each { |rows|
    rows.each { |col, val|
      step %{I set text on "#{col}" with "#{val}"}
    }
  }
end

When /^I run the following steps?:$/ do |steps_table|
  steps = steps_table.raw.flatten
  steps.each do |step|
    call_step step
  end
end

When /^I click on following elements:$/ do |object|
  object.raw.each do |lines|
    lines.each do |line|
      step %{I click on "#{line}"}
    end
  end
end

Then /^I run following steps in iframe (index|name) (.*):$/ do |frame_type, frame, step_table|
  case frame_type.to_sym
    when :index
      steps = step_table.raw.flatten
      browser = page.driver.browser
      browser.switch_to.frame(frame.to_i)
      steps.each do |step|
        call_step step
      end
      browser.switch_to.default_content
    when :name
      browser = page.driver.browser
      browser.switch_to.frame(frame.to_s)
      steps.each do |step|
        call_step step
      end
      browser.switch_to.default_content
  end
end

# get text for object
And /^I get text on "(.*?)" then store it into file "(.*)"$/ do |object, file_name|
  $text = execute_gettext(object)
  open($test_data_dir+file_name, 'a+') do |f|
    f << $text + "\n"
  end
end

# Fill in a text box or text area with a value from $test_data_dir file
And /^I set text on "(.*?)" with value from file "(.*)"$/ do |object, file_name|
  expected_file= ($test_data_dir + file_name)
  if File.exist?(expected_file)
    # data_file = File.read(expected_file)
    data_file = read_file(expected_file)
    execute_settext(object, data_file)
    # data_file.each_line do |line|
    #   execute_settext(object, line)
    # end
  else
    raise "*** ERROR: File #{file_name} is not existed!"
  end
end

# Upload new file windows
When /^I click on "(.*?)" to upload file "(.*?)"$/ do |object, file_name|
  execute_click_to_upload(object, file_name)
end

# Drag mouse hover on element
When /^I move mouse to element "(.*?)"$/ do |element|
  execute_hover_mouse_on(element)
end

And /^I click on "(.*?)" and wait (\d+) seconds$/ do |object, seconds|
  execute_click(object)
  sleep(seconds.to_i)
end

And /^I drag object "(.*?)" to "(.*?)"$/ do |from_object, object|
  execute_drag_to_new_object(from_object, object)
end

# And /^I test$/ do
#   es_data = File.read("C:/Users/Anh Pham/Desktop/testdata_ES_ruby.txt")
#   data = JSON.parse(es_data)
#   data['hits']['hits'].each do |child|
#     replaced_value = child['_source']['timestamp'].gsub("Oct", "2015-10-").gsub!(/\s+/, "").insert(10, 'T')+ ".112Z"
#     new_value = child.to_s.gsub(child['_source']['@timestamp'], "\""+replaced_value+"\"")
#     open("D:/IFD Project Automation/Master Branch/features/MYM/Data/test.txt", 'a+') do |f|
#       f << new_value + "\n"
#     end
#   end
# end

And /^I type to "(.*)" key is "(.*?)"$/ do |object, keys|
  keys = var_collect(keys)
  execute_sendkeys(object, keys)
end

# Click on ID of link or text of link
When /^I click on "([^\"]*)" link$/ do |link|
  click_link link
end

# Upload file
When /^I attach the file "(.*)" to "(.*)"$/ do |filename, field|
  filepath = File.join($base_dir, filename)
  if filepath.match(/^C:\//)
    filepath.gsub!(/\//, '\\')
  end
  attach_file(field, filepath)
end

# double click on web element
Then(/^I double click on element "(.*?)"$/) do |element|
  double_click(element)
end

# step to resize browser
Then(/^I resize browser window size to width (\d+) and height (\d+)$/) do |width, heigth|
  resize_browser(width, heigth)
end

# step to maximize browser
Then(/^I maximize browser window$/) do
  maximize_browser
end

Then(/^I switch to window having title "(.*?)"$/) do |window_title|
  switch_to_window_by_title window_title
end

Then(/^I take screenshot$/) do
  take_screenshot
end

# steps to scroll web page to top or end
Then(/^I scroll to (top|end) of page$/) do |to|
  scroll_page(to)
end
