require 'net/https'
require_relative 'required_files'

# This file contains assertion methods which are called from assertion_steps.rb

# Method to return page title
def get_page_title
  page.title
end

# Method to verify title
# param 1 : String : expected title
# param 2 : Boolean : test case [true or flase]
def check_title(title, test_case)
  page_title = get_page_title
  if test_case
    if page_title != "#{title}"
      raise TestCaseFailed, "Page Title Not Matched, Actual Page Title : #{page_title}, Expected Page Title : #{title}"
    end
  else
    if page_title == "#{title}"
      raise TestCaseFailed, "Page Title Matched, Actual Page Title: #{page_title}"
    end
  end
end

# Method to verify partial title
# param 1 : String : partial title string
# param 2 : Boolean : test case [true or flase]
def check_partial_title(partial_text_title, test_case)
  page_title = get_page_title
  if test_case
    if not page_title.include? "#{partial_text_title}"
      raise TestCaseFailed, 'Partial Page Title Not Present'
    end
  else
    if page_title.include? "#{partial_text_title}"
      raise TestCaseFailed, 'Page Title Matched'
    end
  end
end


# Open Browser with config-able maximize browser option
def execute_openbrowser url_site #, redirect
  begin
    if $_CFWEB['Maximize Browser'] == true
      page.driver.browser.manage.window.maximize
    end
  rescue StandardError => myStandardError
    put_log "\n>>> Error: #{myStandardError}"
  end

  if url_site == ""
    visit("")
  else
    visit(url_site)
  end
end

# Click on element
def execute_click element
  foundElement = find_object(element)
  if foundElement != nil
    page.driver.execute_script('window.focus();')
    startTime = Time.new.to_i
    begin
      sleep(0.5)
      currentTime = Time.new.to_i
    end while (foundElement.native.enabled? == false and (currentTime - startTime) < $_CFWEB['Wait Time'])
    page.driver.browser.execute_script("arguments[0].scrollIntoView(true);", foundElement.native)
    page.driver.browser.mouse.click(foundElement.native)
  else
    put_log "\nError >> Not found object: #{element}"
  end
end

# Set option state with state value True/False
def execute_setstate element, state
  foundElement = find_object(element)
  if foundElement != nil
    if state.upcase == "TRUE"
      foundElement.set(true)
    else
      foundElement.set(false)
    end
  else
    put_log "\nError >> Not found object: #{element}"
    exit
  end
end

#Enter text to element
def execute_settext element, text
  foundElement = find_object(element)
  if foundElement != nil
    begin
      foundElement.set(text)
        #page.driver.browser.mouse.click(foundElement.native)
        #foundElement.native.send_keys(text)
        # put_log "\nfoundElement.value: #{foundElement.value}"
        #foundElement.native.send_keys([:tab])
    rescue StandardError => myStandardError
      put_log "\n>>> Error: #{myStandardError}"
    end
  else
    put_log "\nError >> Not found object: #{element}"
    exit
  end
end

# Move mouse to element then click
def execute_mousehoverandclick element
  foundElement = find_object(element)
  if foundElement != nil
    page.driver.browser.mouse.move_to(foundElement.native, element)
    page.driver.browser.mouse.click(foundElement.native)
    sleep(2)
  else
    put_log "\nError >> Not found object: #{element}"
  end
end

def execute_drag_to_new_object from_element, element
  found_from_element = find_object(from_element)
  foundElement = find_object(element)
  if foundElement != nil and found_from_element != nil
    found_from_element.drag_to(foundElement)
  end
end

def execute_hover_mouse_on element
  foundElement = find_object(element)
  if foundElement != nil
    page.driver.browser.mouse.move_to(foundElement.native, element)
    sleep 1
  else
    put_log "\nError >> Not found object: #{element}"
  end
end

def execute_click_to_upload object, file_name
  upload_file = ($test_data_dir + file_name).gsub!(/\//, "\\")
  foundElement = find_object(object)
  if foundElement != nil
    call_step("I click on \"#{object}\"")
    window = RAutomation::Window.new(:title => /File Upload/)
    window.text_field(:class => "Edit").set(upload_file)
    window.button(:value => "&Open").click
  else
    raise "Error >> Object #{object} not found.!"
  end
end

# Check exists
def execute_checkexists negate, element, timeout
  origin_timeout = $_CFWEB['Wait Time']
  set_time_out(timeout)
  foundElement = find_object(element)
  set_time_out(origin_timeout)
  #put_log "\nexecute_checkexists : #{foundElement}"
  check = false
  if foundElement != nil
    check = true
  end

  negate = negate.strip if negate != nil
  #puts "---------->#{negate}"
  if negate == "not"
    # check.should_not eq true
    expect(check).not_to eql true
  elsif check.should eq true
    expect(check).to eq true
  end
end

# Send Key
def execute_sendkeys element, text
  foundElement = find_object(element)
  if foundElement != nil
    if foundElement[:disabled] == ''
      if text == ":down"
        foundElement.native.send_keys([:down])
      elsif text == ":enter"
        foundElement.native.send_keys([:enter])
      elsif text == ":tab"
        foundElement.native.send_keys([:tab])
      else
        foundElement.native.send_keys(text)
      end
    end
  else
    put_log "\nError >> Not found object: #{element}"
    exit
  end
end

# method to scroll page to top or end
def scroll_page(to)
  if to == 'end'
    page.driver.execute_script('window.scrollTo(0,Math.max(document.documentElement.scrollHeight,document.body.scrollHeight,document.documentElement.clientHeight));')
  elsif to == 'top'
    page.driver.execute_script('window.scrollTo(Math.max(document.documentElement.scrollHeight,document.body.scrollHeight,document.documentElement.clientHeight),0);')
  else
    raise "Exception : Invalid Direction (only scroll \"top\" or \"end\")"
  end
end

# Method to return element status - enabled?
# param 1 : String : Locator type (id, name, class, xpath, css)
# param 2 : String : Locator value
def is_element_enabled(element)
  foundElement = find_object(element)
  expect(foundElement).to be_visible
end

# Element enabled checking
# param 1 : String : Expected element text
# param 2 : Boolean : test case [true or flase]
def check_element_enable(element, test_case)
  result = is_element_enabled(element)

  if test_case
    raise TestCaseFailed, 'Element Not Enabled' unless result
  else
    raise TestCaseFailed, 'Element Enabled' unless !result
  end
end


# method to assert checkbox check/uncheck
# param 1 : String : Locator value
# param 2 : Boolean : test case [true or flase]
def is_checkbox_checked(element, should_be_checked = true)
  foundElement = find_object(element)
  checkbox = expect(foundElement).to be_checked

  if !checkbox && should_be_checked
    raise TestCaseFailed, 'Checkbox is not checked'
  elsif checkbox && !should_be_checked
    raise TestCaseFailed, 'Checkbox is checked'
  end
end

# method to assert radio button selected/unselected
# param 1 : String : Locator value
# param 2 : Boolean : test case [true or flase]
def is_radio_button_selected(element, should_be_selected = true)
  foundElement = find_object(element)
  radio = expect(foundElement).to be_checked

  if !radio && should_be_selected
    raise TestCaseFailed, 'Radio Button not selected'
  elsif radio && !should_be_selected
    raise TestCaseFailed, 'Radio Button is selected'
  end
end

def double_click(element)
  foundElement = find_object(element)
  page.driver.browser.mouse.double_click(foundElement.native)
end

# Method to maximize browser
def maximize_browser
  page.driver.browser.manage.window.maximize
end

# Method to switch to window by title
def switch_to_window_by_title window_title
  $previous_window = page.driver.browser.window_handle
  window_found = false
  page.driver.browser.window_handles.each{ |handle|
    page.driver.browser.switch_to.window handle
    if page.title == window_title
      window_found = true
      break
    end
  }
  raise "Window having title \"#{window_title}\" not found" if not window_found
end

# Method to resize browser
def resize_browser(width, heigth)
  page.driver.browser.manage.window.resize_to(width, heigth)
end

def take_screenshot
  cur_time = Time.now.strftime('%Y%m%d%H%M%S%L')
  page.driver.browser.save_screenshot($test_data_dir + '/screenshots/screenshot' + cur_time + '.png')
end
